// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#include <fstream>
#include <iostream>
#include <set>
#include <sstream>
#include <string>
#include "osmfixer/index/StatIdx.h"

using osmfixer::StatIdx;
using osmfixer::OsmAttrs;
using osmfixer::Station;

// _____________________________________________________________________________
void StatIdx::readFromFile(const std::string& path) {
  std::ifstream f(path);

  std::string line;

  // first, parse nodes
  while (std::getline(f, line)) {
    // empty line is separator between blocks
    if (line.size() == 0) break;

    std::stringstream rec(line);
    size_t osmid;
    double lat, lng;
    OsmAttrs attrs;

    rec >> osmid;
    rec >> lat;
    rec >> lng;

    addStation(osmid, lat, lng, attrs);
  }

  initIndex();

  std::cout << util::geo::getWKT(_bbox) << std::endl;
}

// _____________________________________________________________________________
void StatIdx::addStation(size_t osmid, double lat, double lng,
                         const OsmAttrs& attrs) {
  std::cout << "Record: " << osmid << " " << lat << " " << lng << std::endl;
  auto point = util::geo::latLngToWebMerc<double>(lat, lng);

  _stations.emplace_back(Station{osmid, point, attrs});

  // extend bounding box
  _bbox = util::geo::extendBox(point, _bbox);
}

// _____________________________________________________________________________
void StatIdx::initIndex() {
  _grid = util::geo::Grid<size_t, util::geo::Point, double>(5000, 5000, _bbox,
                                                            false);
  for (size_t i = 0; i < _stations.size(); i++) {
    _grid.add(_stations[i].pos, i);
  }
}

// _____________________________________________________________________________
std::vector<const Station*> StatIdx::getStations(
    const util::geo::DBox bbox) const {
  std::vector<const Station*> ret;
  auto ll = util::geo::latLngToWebMerc<double>(bbox.getLowerLeft().getX(),
                                               bbox.getLowerLeft().getY());
  auto ur = util::geo::latLngToWebMerc<double>(bbox.getUpperRight().getX(),
                                               bbox.getUpperRight().getY());

  std::set<size_t> tmp;
  auto reqBox = util::geo::DBox(ll, ur);
  _grid.get(reqBox, &tmp);

  for (auto i : tmp) {
    if (util::geo::contains(_stations[i].pos, reqBox))
      ret.push_back(&_stations[i]);
  }

  return ret;
}
