// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#include "osmfixer/server/StatServer.h"
#include "util/Misc.h"
#include "util/String.h"
#include "util/geo/Geo.h"
#include "util/log/Log.h"
#include "util/String.h"

using osmfixer::StatServer;
using osmfixer::Params;

// _____________________________________________________________________________
util::http::Answer StatServer::handle(const util::http::Req& req,
                                      int con) const {
  UNUSED(con);
  util::http::Answer a;
  try {
    Params params;
    auto cmd = parseUrl(req.url, req.payload, &params);

    if (cmd == "/") {
      a = util::http::Answer("200 OK", "osmfixer");
    } else if (cmd == "/map") {
      a = handleMapReq(params);
    } else {
      a = util::http::Answer("404 Not Found", "dunno");
    }
  } catch (std::runtime_error e) {
    a = util::http::Answer("400 Bad Request", e.what());
  } catch (std::invalid_argument e) {
    a = util::http::Answer("400 Bad Request", e.what());
  } catch (...) {
    a = util::http::Answer("500 Internal Server Error",
                           "Internal Server Error.");
  }

  a.params["Access-Control-Allow-Origin"] = "*";
  a.params["Server"] = "osmfixer";

  return a;
}

// _____________________________________________________________________________
util::http::Answer StatServer::handleMapReq(const Params& pars) const {
  if (pars.count("bbox") == 0 || pars.find("bbox")->second.empty())
    throw std::invalid_argument("No bbox specified.");
  std::string cb;
  if (pars.count("cb")) cb = pars.find("cb")->second.c_str();
  auto box = util::split(pars.find("bbox")->second, ',');

  if (box.size() != 4)
    throw std::invalid_argument("Invalid request.");

  double lat1 = atof(box[0].c_str());
  double lng1 = atof(box[1].c_str());
  double lat2 = atof(box[2].c_str());
  double lng2 = atof(box[3].c_str());

  std::cout << pars.find("bbox")->second << std::endl;

  util::geo::DBox bbox(util::geo::DPoint(lng1, lat1), util::geo::DPoint(lng2, lat2));

  LOG(INFO) << "Request for bounding box " << util::geo::getWKT(bbox);

  std::stringstream json;

  if (cb.size()) json << cb << "(";
  json << "[";

  // TODO: query index

  json << "]";
  if (cb.size()) json << ")";

  auto answ = util::http::Answer("200 OK", json.str(), true);
  answ.params["Content-Type"] = "application/javascript; charset=utf-8";

  return answ;
}

// _____________________________________________________________________________
std::string StatServer::parseUrl(std::string u, std::string pl,
                                 std::map<std::string, std::string>* params) {
  auto parts = util::split(u, '?');

  if (parts.size() > 1) {
    auto kvs = util::split(parts[1], '&');
    for (auto kv : kvs) {
      auto kvp = util::split(kv, '=');
      if (kvp.size() == 1) kvp.push_back("");
      (*params)[util::urlDecode(kvp[0])] = kvp[1];
    }
  }

  // also parse post data
  auto kvs = util::split(pl, '&');
  for (auto kv : kvs) {
    auto kvp = util::split(kv, '=');
    if (kvp.size() == 1) kvp.push_back("");
    (*params)[util::urlDecode(kvp[0])] = kvp[1];
  }

  return util::urlDecode(parts.front());
}
